package ui.view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JToolTip;

import com.sun.javafx.geom.Edge;

import classes.CpsNode;
import classes.CpsObject;
import classes.HolonElement;
import classes.HolonObject;
import classes.HolonSwitch;
import javafx.util.Pair;
import ui.controller.Control;
import ui.model.*;

class MyCanvas extends JPanel implements MouseListener, MouseMotionListener {
	private Image img = null; // Contains the image to draw on MyCanvas
	private int x = 0;
	private int y = 0;
	// edge Object Start Point
	private Model model;
	private final Control controller;
	Graphics2D g2; // For Painting
	private int cx;
	private int cy;
	ArrayList<HolonElement> dataSelected = new ArrayList<HolonElement>();

	private boolean dragging = false; // for dragging
	private boolean drawEdge = false; // for drawing edges
	private boolean click = false; // for double click
	public CpsObject tempCps = null;
	private Rectangle selectRect = new Rectangle();

	// PopUpMenu
	private JPopupMenu popmenu = new JPopupMenu();
	private JMenuItem itemDelete = new JMenuItem("Delete Object");
	private JToolTip objectTT = new JToolTip();

	public MyCanvas(final Model model, Control control) {
		this.add(objectTT);
		this.controller = control;
		this.model = model;
		popmenu.add(itemDelete);
		itemDelete.setEnabled(false);
		itemDelete.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				// Remove the selected Object object
				controller.deleteObjectOnCanvas(tempCps);
				tempCps = null;
				selectRect.setRect(0, 0, 0, 0);
				repaint();
			}
		});

		this.addMouseListener(this);
		this.addMouseMotionListener(this);
	}

	/**
	 * Paints all Components on the Canvas
	 * 
	 * @param Graphics
	 * 
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		// Rendering
		g2 = (Graphics2D) g;
		RenderingHints rh = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setRenderingHints(rh);

		// Selection
		if (selectRect != null) {
			g2.setColor(Color.GREEN);
			g2.fillRect((int) selectRect.getX(), (int) selectRect.getY(), (int) selectRect.getWidth(),
					(int) selectRect.getHeight());
		}

		// drawEdges
		g2.setColor(Color.BLACK);
		g2.setStroke(new BasicStroke(2));
		if (drawEdge)
			g2.drawLine(tempCps.getPosition().x + controller.getScaleDiv2(),
					tempCps.getPosition().y + controller.getScaleDiv2(), x, y);

		for (CpsObject cps : model.getObjectsOnCanvas()) {
			for (CpsObject con : cps.getConnectedTo()) {
				if (con.getID() != model.getSelectedObjectID() && cps.getID() != model.getSelectedObjectID())
					g2.drawLine(cps.getPosition().x + controller.getScaleDiv2(),
							cps.getPosition().y + controller.getScaleDiv2(),
							con.getPosition().x + controller.getScaleDiv2(),
							con.getPosition().y + controller.getScaleDiv2());
				g2.drawString("100", (cps.getPosition().x+con.getPosition().x)/2+model.getScaleDiv2(), (cps.getPosition().y+con.getPosition().y)/2+model.getScaleDiv2());
			}
		}

		// Highlighted Edge
		g2.setColor(Color.GREEN);
		for (CpsObject cps : model.getObjectsOnCanvas()) {
			for (CpsObject con : cps.getConnectedTo()) {
				if (con.getID() == model.getSelectedObjectID())
					g2.drawLine(cps.getPosition().x + controller.getScaleDiv2(),
							cps.getPosition().y + controller.getScaleDiv2(),
							con.getPosition().x + controller.getScaleDiv2(),
							con.getPosition().y + controller.getScaleDiv2());
			}
		}

		
		
		// Objects
		for (CpsObject cps : model.getObjectsOnCanvas()) {
			img = new ImageIcon(this.getClass().getResource(cps.getImage())).getImage();

			g2.drawImage(img, cps.getPosition().x, cps.getPosition().y, controller.getScale(), controller.getScale(),
					null);

		}
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		// If double clicked on a Switch change the Image to on/off
		if (doubleClick() && tempCps != null && tempCps.getClass() == HolonSwitch.class) {
			System.out.println("trans double click");
			if (tempCps.getImage().compareTo("/Images/switch-on.png") == 0) {
				tempCps.setImage("/Images/switch-off.png");
			} else {
				tempCps.setImage("/Images/switch-on.png");
			}
		}
		repaint();
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mousePressed(MouseEvent e) {
		tempCps = null;
		dataSelected = null;
		// Object Selection
		for (CpsObject cps : model.getObjectsOnCanvas()) {
			cx = cps.getPosition().x;
			cy = cps.getPosition().y;
			if (x - controller.getScale() <= cx && y - controller.getScale() <= cy && x >= cx && y >= cy) {
				tempCps = cps;
				// If drawing an Edge (CTRL down)
				if (e.isControlDown()) {
					drawEdge = true;
					/*
					controller.deleteElementCanvas(1, "Rehab");
					System.out.println(((HolonObject) model.getCategories().get(0).getObjects().get(0)).getElements()
							.get(2).getAmount());
					*/
				}
				if (tempCps.getClass() == HolonObject.class) {
					HolonObject tempObj = ((HolonObject) tempCps);
					dataSelected = tempObj.getElements();
				}
			}
		}
		// Object Selection Highlighting (selectRect)
		objectSelectionHighlighting();

		repaint();
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		if (drawEdge) {
			drawEdge = false;
			drawDeleteEdge();
		}

		// if Dragged reposition the Object
		if (dragging) {
			x = e.getX();
			y = e.getY();
			dragging = false;
			tempCps.setPosition(e.getX() - controller.getScaleDiv2(), e.getY() - controller.getScaleDiv2());
		}

		// Rightclick List
		if (e.getButton() == e.BUTTON3) {
			if (e.getButton() == e.BUTTON3 && tempCps != null) {
				itemDelete.setEnabled(true);
			} else {
				itemDelete.setEnabled(false);
			}
			popmenu.show(e.getComponent(), e.getX(), e.getY());
		}

		repaint();
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		// If Edge is drawn
		if (drawEdge) {
			x = e.getX();
			y = e.getY();
			repaint();
		} else {
			try {
				// Au�erhalb des Randes gedragged?
				x = e.getX() - controller.getScaleDiv2();
				y = e.getY() - controller.getScaleDiv2();
				if (e.getX() < controller.getScaleDiv2())
					x = 0;
				else if (e.getX() > this.getWidth() - controller.getScaleDiv2())
					x = this.getWidth() - controller.getScale();
				if (e.getY() < controller.getScaleDiv2())
					y = 0;
				else if (e.getY() > this.getHeight() - controller.getScaleDiv2())
					y = this.getHeight() - controller.getScale();
				// Drag Position
				tempCps.setPosition(x, y);
				// Highlighting Position
				selectRect.setLocation(x - (controller.getScale() / 20), y - (controller.getScale() / 20));
				// TipText Position and name
				objectTT.setTipText(tempCps.getName());
				objectTT.setLocation(x, y + controller.getScale());
				repaint();
			} catch (Exception e2) {

			}
		}
	}

	@Override
	public void mouseMoved(MouseEvent e) {
		x = e.getX();
		y = e.getY();
		// Everytghing for the tooltip :)
		boolean on = false;
		for (CpsObject cps : model.getObjectsOnCanvas()) {

			cx = cps.getPosition().x;
			cy = cps.getPosition().y;
			if (x - controller.getScale() <= cx && y - controller.getScale() <= cy && x >= cx && y >= cy) {

				objectTT.setTipText(cps.getName());
				objectTT.setLocation(cx, cy + controller.getScale());
				on = true;
			}
		}
		if (!on) {
			objectTT.setLocation(-200, -200);
			objectTT.setTipText("");
		}
	}

	/**
	 * Sets the Highlighting of the Selected Object
	 */
	public void objectSelectionHighlighting() {
		if (tempCps != null) {
			selectRect.setBounds(tempCps.getPosition().x - (controller.getScale() / 20),
					tempCps.getPosition().y - (controller.getScale() / 20),
					controller.getScale() + controller.getScale() / 10,
					controller.getScale() + controller.getScale() / 10);

			controller.setSelectedObjectID(tempCps.getID());
		} else {
			controller.setSelectedObjectID(0);
			selectRect.setRect(0, 0, 0, 0);
		}
	}

	/**
	 * Draws or Deletes an Edge
	 */
	private void drawDeleteEdge() {
		boolean node = true;
		CpsNode n = null;
		for (CpsObject cps : model.getObjectsOnCanvas()) {
			cx = cps.getPosition().x;
			cy = cps.getPosition().y;
			if (x - controller.getScale() <= cx && y - controller.getScale() <= cy && x >= cx && y >= cy && cps != tempCps) {
				node = false;
				if (!cps.getConnectedTo().contains(tempCps)) {
					cps.AddConnection(tempCps);
					tempCps.AddConnection(cps);
				} else {
					cps.getConnectedTo().remove(tempCps);
					tempCps.getConnectedTo().remove(cps);
				}
			}
		}
		if(node){			
			n = new CpsNode("Node");
			n.setPosition(x-model.getScaleDiv2(),y-model.getScaleDiv2());
			controller.addObjectCanvas(n);
			n.AddConnection(tempCps);
			tempCps.AddConnection(n);
			System.out.println("node ID: "+n.getID());
		}
	}

	/**
	 * Checks if a double click was made
	 * 
	 * @return true if doublecklick, false if not
	 */
	private boolean doubleClick() {
		if (click) {
			click = false;
			return true;
		} else {
			click = true;
			Timer t = new Timer("doubleclickTimer", false);
			t.schedule(new TimerTask() {
				@Override
				public void run() {
					click = false;
				}
			}, 500);
		}
		return false;
	}
}